﻿using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Layouts;
using ArcGIS.Desktop.Mapping;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;

namespace TrayButtonDemo.Map
{
  internal class MapTrayLabelingPopup : MapTrayButton
  {
    /// <summary>
    /// Invoked after construction, and after all DAML settings have been loaded. 
    /// Use this to perform initialization such as setting ButtonType.
    /// </summary>
    protected override void Initialize()
    {
      base.Initialize();

      // set the button type
      //  change for different button types
      ButtonType = TrayButtonType.PopupToggleButton;

    }
    /// <summary>
    /// Construct the popup view and return it. 
    /// </summary>
    /// <returns></returns>
    protected override ContentControl ConstructPopupContent()
    {
      //TODO: Good spot to initialize, defaults?
      string defaultFontName = null;
      if (Module1.AvailableFontsStylesMapping.Count > 0)
      {
        if (Module1.AvailableFontsStylesMapping.ContainsKey("Arial Black"))
          defaultFontName = "Arial Black";
      }
      // set up the tray button ViewModel
      _popupVM = new MapTrayLabelingPopupViewModel()
      {
        IsChecked = this.IsChecked,
        CanPopupAutoClose = this.CanAutoClose,
        SelectedColor = ColorFactory.Instance.BlackRGB,
        SelectedFontSize = 12.0,
        SelectedFontName = defaultFontName
      };

      // return the UI with the datacontext set
      return new MapTrayLabelingPopupView()
      {
        DataContext = _popupVM
      };
    }
    #region TrayButtonType.ToggleButton / TrayButtonType.PopupToggleButton

    /// <summary>
    /// Called when the toggle button check state changes
    /// </summary>
    protected override void OnButtonChecked()
    {
      // get the checked state
      var isChecked = this.IsChecked;

      #region refresh the popup VM checked state
      if (_popupVM == null) return;

      // refresh the popup VM checked state
      if ((_popupVM != null) && (_popupVM.IsChecked != this.IsChecked))
        _popupVM.IsChecked = this.IsChecked;
      #endregion

      //Get all layers in the map
      var featureLayers = MapView.Active.Map.GetLayersAsFlattenedList()
                          .OfType<FeatureLayer>().ToList();
      if (featureLayers.Count == 0)
        return;//nothing to do
      //Apply labels
      QueuedTask.Run(() =>
      {
        foreach (var featurelayer in featureLayers)
        {
          _popupVM.LabelLayers(featurelayer, isChecked);
        }
      });
    }
    #endregion
    /// <summary>
    /// Override to perform some button initialization.  This is called the first time the botton is loaded.
    /// </summary>
    protected override void OnButtonLoaded()
    {
      base.OnButtonLoaded();
    }




    #region TrayButtonType.PopupToggleButton

    // These methods fire when ButtonType = TrayButtonType.PopupToggleButton

    private MapTrayLabelingPopupViewModel _popupVM = null;

   

    private bool _subscribed = false;

    /// <summary>
    /// Called when the popup is shown. 
    /// </summary>
    protected override void OnShowPopup()
    {

      // track property changes
      if (!_subscribed)
      {
        _popupVM.PropertyChanged += MapTrayLabelingPopupViewModel_PropertyChanged;
        _subscribed = true;
      }
    }

    /// <summary>
    /// Called when the popup is hidden.
    /// </summary>
    protected override void OnHidePopup()
    {
      // cleanup
      if (_subscribed)
      {
        _popupVM.PropertyChanged -= MapTrayLabelingPopupViewModel_PropertyChanged;
        _subscribed = false;
      }
      base.OnHidePopup();
    }

    private void MapTrayLabelingPopupViewModel_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
    {
      if (_popupVM == null)
        return;
      // make sure MapTrayButton class has correct checked state when it changes on the VM
      if (e.PropertyName == nameof(MapTrayLabelingPopupViewModel.IsChecked))
      {
        // Since we are changing IsChecked in OnButtonChecked
        //We don't want property notification to trigger (another) callback to OnButtonChecked
        this.SetCheckedNoCallback(_popupVM.IsChecked);
        #region Label Feature
        //Get all layers in the map
        var featureLayers = MapView.Active.Map.GetLayersAsFlattenedList()
                            .OfType<FeatureLayer>().ToList();
        if (featureLayers.Count == 0)
          return;//nothing to do
                 //Apply labels
        QueuedTask.Run(() =>
        {
          foreach (var featurelayer in featureLayers)
          {
            _popupVM.LabelLayers(featurelayer, (_popupVM.IsChecked));
          }
        });
        #endregion
      }
      //Control auto close behavior of tray button 
      else if (e.PropertyName == nameof(MapTrayLabelingPopupViewModel.CanPopupAutoClose))
      {
        this.CanAutoClose = _popupVM.CanPopupAutoClose;
      }

    }

    // Provided to show you how to manually close the popup via code. 
    private void ManuallyClosePopup()
    {
      this.ClosePopup();
    }
    #endregion
  }
}
