﻿using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Internal.Mapping.Symbology;
using ArcGIS.Desktop.Layouts;
using ArcGIS.Desktop.Mapping;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Data;

namespace TrayButtonDemo.Map
{
  internal class MapTrayLabelingPopupViewModel : PropertyChangedBase
  {
    private static readonly object _fontStylesLock = new object();
    public MapTrayLabelingPopupViewModel()
    {
      BindingOperations.EnableCollectionSynchronization(Module1.Current.AvailableFontStyles, _fontStylesLock);
    }
    /// <summary>
    /// Text shown near the top Map Tray UI.
    /// </summary>
    private string _heading =  "Set options for text labels";
    public string Heading
    {
      get => _heading;
      set => SetProperty(ref _heading, value);
    }

    private bool _isChecked;
    public bool IsChecked
    {
      get => _isChecked;
      set => SetProperty(ref _isChecked, value);
    }

    public ObservableCollection<string> FontNames => Module1.Current.AvailableFontNames;

    private string _selectedFontName; 
    public string SelectedFontName
    {
      get => _selectedFontName;
      set
      {
        lock (_fontStylesLock)
        {
          Module1.Current.AvailableFontStyles.Clear();
          foreach (var style in Module1.AvailableFontsStylesMapping[value])
            Module1.Current.AvailableFontStyles.Add(style);
          SelectedFontStyle = "Regular";
          if (!Module1.Current.AvailableFontStyles.Contains(_selectedFontStyle))
            SelectedFontStyle = Module1.Current.AvailableFontStyles[0];
        }
        SetProperty(ref _selectedFontName, value);
      }
    }

    public ObservableCollection<string> FontStyles => Module1.Current.AvailableFontStyles;

    private string _selectedFontStyle;
    public string SelectedFontStyle
    {
      get => _selectedFontStyle;
      set => SetProperty(ref _selectedFontStyle, value);
    }

    private CIMColor _selectedColor; 
    public CIMColor SelectedColor
    {
      get => _selectedColor;
      set => SetProperty(ref _selectedColor, value);
    }

    private double _selectedFontSize;
    public double SelectedFontSize
    {
      get => _selectedFontSize;
      set => SetProperty(ref _selectedFontSize, value);
    }

    private bool _includeHalo;
    public bool IncludeHalo
    {
      get => _includeHalo;
      set => SetProperty(ref _includeHalo, value);
    }
    #region Event Handlers
    //Property to control auto close behavior of tray button 
    private bool _canPopupAutoClose;
    public bool CanPopupAutoClose
    {
      get => _canPopupAutoClose;
      set => SetProperty(ref _canPopupAutoClose, value);
    }
    private CIMTextSymbol _textSymbol;

    internal void UpdateTextSymbol()
    {
      //Create TextSymbol
      _textSymbol = SymbolFactory.Instance.ConstructTextSymbol(SelectedColor, SelectedFontSize, SelectedFontName, SelectedFontStyle);
      if (IncludeHalo)
      {
        var halo = SymbolFactory.Instance.ConstructPolygonSymbol(
            ColorFactory.Instance.WhiteRGB, SimpleFillStyle.Solid, null);
        _textSymbol.HaloSize = 1.0;
        _textSymbol.HaloSymbol = halo;
      }
    }
    internal void LabelLayers(FeatureLayer featureLayer, bool isChecked)
    {
      if (isChecked)
      {
        //Construct text symbol using the properties
        _textSymbol = SymbolFactory.Instance.ConstructTextSymbol(SelectedColor, SelectedFontSize, SelectedFontName, SelectedFontStyle);
# region Include Halo?
        if (IncludeHalo)
        {
          var halo = SymbolFactory.Instance.ConstructPolygonSymbol(
              ColorFactory.Instance.WhiteRGB, SimpleFillStyle.Solid, null);
          _textSymbol.HaloSize = 1.0;
          _textSymbol.HaloSymbol = halo;
        }
        #endregion
        //Get the layer's LabelClass
        var labelClass = featureLayer.LabelClasses.FirstOrDefault();
        if (labelClass != null)
        {
          //Sets the label class text symbol
          labelClass.SetTextSymbol(_textSymbol);
        }
      }
      //Toggle layer visibility
      featureLayer.SetLabelVisibility(isChecked);
    }
    #endregion
  }
}
